/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2012, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/** \file     TComAdaptiveFilter.h
    \brief    adaptive filter class (header)
*/
#ifndef __TCOMADAPTIVEFILTER__
#define __TCOMADAPTIVEFILTER__

#include "CommonDef.h"
#include "TComPic.h"

#if SVC_IL_ADAPTIVE_FILTER

//! \ingroup TLibCommon
//! \{

// ====================================================================================================================
// function definition
// ====================================================================================================================

// ====================================================================================================================
// constant definition
// ====================================================================================================================
#define AF_COEF_PRECISION_BITS     7            //Unsigned N-bits for center coefficient; signed N-bits for non-center coefficient
#define AF_NUM_BIT_SHIFT          (Int)(AF_COEF_PRECISION_BITS-1)    ///< bit shift parameter for quantization of ALF param.

// ====================================================================================================================
// struct/enum
// ====================================================================================================================

enum AF_COMPONENT_SYMBOL
{
  AF_Y =0,
  AF_Cb,
  AF_Cr,
  AF_NUM_COMPONETS
};

enum AF_FILTER_SHAPE
{
   AF_SHAPE_7x7CROSS_3x3SQUARE = 0
  ,AF_NUM_FILTER_SHAPE
};


// ====================================================================================================================
// Global variables
// ====================================================================================================================
extern Int* g_AFNumCoef;

// ====================================================================================================================
// class
// ====================================================================================================================

class TComAdaptiveFilter
{
public: 
  TComAdaptiveFilter();
  ~TComAdaptiveFilter();

  Void create( Int picWidth, Int picHeight, UInt maxCUWidth, UInt maxCUHeight, UInt maxCUDepth );
  Void destroy(); 

public:  
  Void filterBlock(Int shape, Int* coef, Pel* srcBlk, Pel* resBlk, Int srcStride, Int resStride, Int width, Int height);
  Void reconstructCoef(Int* dstCoef, Int* srcCoef, Int shape =0 ); 
protected:
  Pel* getPicBuf(TComPicYuv* pPicYuv, Int compIdx);
  Void checkFilterCoeffValue( Int *coef, Int filterLength);
  Void createFilterShapeInfo();
  Void destroyFilterShapeInfo();
protected:

  //size-related 
  Int m_picWidth;
  Int m_picHeight;
  Int m_maxCUWidth;
  Int m_maxCUHeight;
  Int m_maxCUDepth;
  Int m_numCTUInWidth;
  Int m_numCTUInHeight;

  //filter shapes
  Int* m_shapeWeight[AF_NUM_FILTER_SHAPE]; 

  //temporary buffer
  TComPicYuv*   m_tempPicYuv;

private:
};


//------

#endif
#endif
